#!/usr/bin/env python
# coding: utf-8

# Copyright (c) Simula Research Laboratory.
# Distributed under the terms of the Modified BSD License.

from __future__ import print_function

import os
from pathlib import PurePath

from globmatch import glob_match


def check_dirs(source_dir, build_dir, known_excludes=None):
    """Run a check of a generated package manifest.

    The generated manifest will be checked against its source directory,
    and any unincluded files will be reported.  Any files declared
    explicitly in MAINFEST.in that are missing will also be reported.
    Any files that should intentionally not be included should be passed
    in the known_excludes argument.
    """

    return check_iterables(
        filter(os.path.isfile, os.walk(source_dir)),
        filter(os.path.isfile, os.walk(build_dir)),
        known_excludes=known_excludes,
        )


def check_filelist(filelist, known_excludes=None):
    """Check the files found in a FileList.

    filelist: distutils.FileList
        The file list object generated by a distutil command.
    """
    if filelist.allfiles is None:
        filelist.findall()
    return check_iterables(
        filelist.allfiles,
        filelist.files,
        known_excludes=known_excludes,
        )


def check_iterables(source, dist, known_excludes=None):
    """Check the files in the two iterables against each other.

    Any extra files in dist will be reported unconditionally, while any extra
    files in source will be reported unless it matches a glob in known_excludes.

    A report is made using print().

    Returns:
        A bitfield value with the following values:
            == 0 if no extra files were found
            |= 1 if extra files were found in dist
            |= 2 if extra files were found in source that were not excluded
    """
    source = set(PurePath(p) for p in source)
    dist = set(PurePath(p) for p in dist)

    retval = 0

    for missing in dist - source:
        retval |= 1
        print('Missing file: %r' % str(missing))

    for extra in filter(lambda x: not glob_match(x, known_excludes), source - dist):
        retval |= 2
        print('Extra file: %r' % str(extra))

    return retval



#!/usr/bin/env python
# coding: utf-8

# Copyright (c) Simula Research Laboratory.
# Distributed under the terms of the Modified BSD License.

"""Distutils commands for checking generated files"""

from distutils.core import Command

from .check import check_filelist

def manifix_sdist_command(original_command=None, known_excludes=None, callback=None):
    """Create a manifix setup sdist command

    Parameters
    ----------
    original_command: Command class
        The original command to override. Defaults to setuptools.
    known_excludes:

    callback: function, optional
        Override for the callback to call with the generated file list.
    """

    if original_command is None:
        from setuptools.command.sdist import sdist
        original_command = sdist
    if callback is None:
        callback = check_filelist

    class ManifixSdistCommand(original_command):
        def initialize_options(self):
            original_command.initialize_options(self)
            self.known_excludes = None

        def finalize_options(self):
            original_command.finalize_options(self)
            self.known_excludes = self.known_excludes.splitlines() or []
            self.known_excludes.extend(known_excludes or [])
            self.known_excludes = [e for e in self.known_excludes if e]

        def make_distribution(self):
            ret = callback(self.filelist, known_excludes=self.known_excludes)
            self.archive_files = []
            if ret:
                raise RuntimeError('Manifix detected some errors, see output for details')

    return ManifixSdistCommand


class DefaultManifixSdistCommand(Command):
    """A manifix command that calls the registered sdist internally"""
    user_options = []

    def initialize_options(self):
        self.known_excludes = None

    def finalize_options(self):
        if self.known_excludes is None:
            # Default behaviour to exclude VCS folders
            self.known_excludes = ['.git', '.hg']
        else:
            self.known_excludes = self.known_excludes.splitlines()
        self.known_excludes = [e for e in self.known_excludes if e]

    def run(self):
        sdist = self.get_finalized_command('sdist')

        def make_distribution(this):
            ret = check_filelist(this.filelist, known_excludes=self.known_excludes)
            this.archive_files = []
            if ret:
                raise RuntimeError('Manifix detected some errors, see output for details')

        import types
        sdist.make_distribution = types.MethodType(make_distribution, sdist)
        self.run_command('sdist')



#!/usr/bin/env python
# coding: utf-8

import io
import os
import sys

from setuptools import setup, find_packages


here = os.path.abspath(os.path.dirname(__file__))

# the name of the project
name = 'manifix'

version_ns = {}
with io.open(os.path.join(here, name, '_version.py'), encoding="utf8") as f:
    exec(f.read(), {}, version_ns)


setup_args = dict(
    name            = name,
    description     = "Check for any extra/missing files for distribution",
    version         = version_ns['__version__'],
    packages        = find_packages(here),
    author          = 'Vidar Tonaas Fauske',
    author_email    = 'vidartf@gmail.com',
    license         = 'BSD-3',
    platforms       = "Linux, Mac OS X, Windows",
    keywords        = ['distribution', 'manifest', 'package'],
    python_requires = '>=3.6',
    classifiers     = [
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
    ],
)


setuptools_args = {}
install_requires = setuptools_args['install_requires'] = [
    'globmatch',
    'setuptools',
]

extras_require = setuptools_args['extras_require'] = {
    'test': [
        'pytest',
        'pytest-cov',
    ],
    'docs': [
        'sphinx',
        'recommonmark',
        'sphinx_rtd_theme'
    ],
}

if 'setuptools' in sys.modules:
    setup_args.update(setuptools_args)

    # force entrypoints with setuptools (needed for Windows, unconditional because of wheels)
    setup_args['entry_points'] = {
        "distutils.commands": [
            "manifix = manifix.command:DefaultManifixSdistCommand",
        ],
    }
    setup_args.pop('scripts', None)

    setup_args.update(setuptools_args)

if __name__ == '__main__':
    setup(**setup_args)
